( cd usr/bin ; rm -rf lzegrep )
( cd usr/bin ; ln -sf lzgrep lzegrep )
( cd usr/bin ; rm -rf lzfgrep )
( cd usr/bin ; ln -sf lzgrep lzfgrep )
( cd usr/bin ; rm -rf unlzma )
( cd usr/bin ; ln -sf lzma unlzma )
( cd usr/bin ; rm -rf lzless )
( cd usr/bin ; ln -sf lzmore lzless )
( cd usr/bin ; rm -rf lzcat )
( cd usr/bin ; ln -sf lzma lzcat )
( cd usr/bin ; rm -rf lzcmp )
( cd usr/bin ; ln -sf lzdiff lzcmp )
( cd usr/lib ; rm -rf liblzmadec.so.0 )
( cd usr/lib ; ln -sf liblzmadec.so.0.0.0 liblzmadec.so.0 )
( cd usr/lib ; rm -rf liblzmadec.so )
( cd usr/lib ; ln -sf liblzmadec.so.0.0.0 liblzmadec.so )
( cd usr/man/man1 ; rm -rf lzcat.1.gz )
( cd usr/man/man1 ; ln -sf lzma.1.gz lzcat.1.gz )
( cd usr/man/man1 ; rm -rf lzfgrep.1.gz )
( cd usr/man/man1 ; ln -sf lzgrep.1.gz lzfgrep.1.gz )
( cd usr/man/man1 ; rm -rf lzcmp.1.gz )
( cd usr/man/man1 ; ln -sf lzdiff.1.gz lzcmp.1.gz )
( cd usr/man/man1 ; rm -rf lzless.1.gz )
( cd usr/man/man1 ; ln -sf lzmore.1.gz lzless.1.gz )
( cd usr/man/man1 ; rm -rf unlzma.1.gz )
( cd usr/man/man1 ; ln -sf lzma.1.gz unlzma.1.gz )
( cd usr/man/man1 ; rm -rf lzegrep.1.gz )
( cd usr/man/man1 ; ln -sf lzgrep.1.gz lzegrep.1.gz )
